@@ -41,23 +41,18 @@ typedef struct {
 	
 	gint64 start_sector;
 	gint64 end_sector;
+	gint msf_minutes;
+	gint msf_seconds;
+	
 	gint64 sectors;
 	gint64 start_time;
 	gint64 end_time;
+	
+	gboolean is_data;
+	gboolean is_lead_out;
 } CdTrackInfo;
 
-typedef struct {
-	gchar *device_node;
-	gchar *disk_id;
-
-	gint64 n_tracks;
-	gint64 total_sectors;
-	gint64 total_time;
-	
-	CdTrackInfo **tracks;
-} CdDiskInfo;
-
-CdDiskInfo *cd_disk_info_new(const gchar *device_node);
-void cd_disk_info_free(CdDiskInfo *disk);
+char *
+cd_info_get_extended_disc_id(char *device);
 
 #endif /* CD_INFO_H */
