@@ -107,9 +107,8 @@ namespace Banshee
 		
 		private TrackInfo track;
 		
-		private bool finalized = false;
-		
-		private bool timeoutCancelRequest = false;
+		private bool finalized;
+		private bool timeoutCancelRequest;
 		
 		public void Initialize()
 		{
@@ -119,6 +118,9 @@ namespace Banshee
 			//gpe_set_error_handler(handle, OnError);
 			//gpe_set_iterate_handler(handle, OnIterate);
 			
+			timeoutCancelRequest = false;
+			finalized = false;
+			
 			GLib.Timeout.Add(250, OnTimeout);
 		}
 		
@@ -151,12 +153,7 @@ namespace Banshee
 				return false;
 			}
 			
-			string uri = ti.Uri;
-			
-			if(!uri.StartsWith("file://") && !uri.StartsWith("cdda://"))
-				 uri = "file://" + uri;
-			
-			loaded = gpe_open(handle, uri);
+			loaded = gpe_open(handle, ti.Uri.AbsoluteUri);
 			
 			if(loaded)
 				track = ti;
