@@ -148,7 +148,7 @@ namespace Banshee
 				return false;
 			}
 			
-			loaded = player.OpenUri("file://" + ti.Uri);
+			loaded = player.OpenUri(ti.Uri.AbsoluteUri);
 			
 			if(loaded)
 				track = ti;
