@@ -29,14 +29,17 @@
  
 using System;
 using System.Collections;
+using System.Threading;
 using System.IO;
-using Mono.Posix;
+using Mono.Unix;
 using IPod;
+using Gtk;
 
 namespace Banshee
 {
     public delegate void IpodDeviceAddedHandler(object o, IpodDeviceArgs args);
     public delegate void IpodDeviceRemovedHandler(object o, IpodDeviceArgs args); 
+    public delegate void IpodDeviceHandler(object o, IpodDeviceArgs args);
 
     public class IpodDeviceArgs
     {
@@ -57,7 +60,6 @@ namespace Banshee
 	
 		public IpodCore()
 		{
-			IPod.Initializer.UseDefaultContext = true;
 			listener = new DeviceEventListener();
 			listener.DeviceAdded += OnDeviceAdded;
 			listener.DeviceRemoved += OnDeviceRemoved;
@@ -128,12 +130,14 @@ namespace Banshee
             return false;
         }
         
-        public static string GetIpodSong(string filename)
+        public static string GetIpodSong(Uri uri)
         {
+            string filename = uri.LocalPath;
+        
             if(ValidSongFormat(filename)) 
                 return filename;
                 
-            string path = Library.MakeFilenameKey(filename);
+            string path = Library.MakeFilenameKey(uri);
             string dir = Path.GetDirectoryName(path);
             string file = Path.GetFileNameWithoutExtension(filename);
             
@@ -155,9 +159,11 @@ namespace Banshee
             return null;
         }
         
-        public static string ConvertSongName(string filename, string newext)
+        public static string ConvertSongName(Uri uri, string newext)
         {
-            string path = Library.MakeFilenameKey(filename);
+            string filename = uri.LocalPath;
+            
+            string path = Library.MakeFilenameKey(uri);
             string dir = Path.GetDirectoryName(path);
             string file = Path.GetFileNameWithoutExtension(filename);
             
@@ -166,234 +172,203 @@ namespace Banshee
         }
 	}
 	
-	public class IpodSyncTransaction : LibraryTransaction
-	{
-		public override string Name {
-			get {
-				return "iPod Sync Transaction";
-			}
-		}
-		
-		private PipelineProfile profile;
-		private FileEncoder encoder;
-		private Device device;
-		private ArrayList updateTracks = null;
-		private ArrayList removeTracks = null;
-		
-		public event EventHandler SyncStarted;
-		public event EventHandler SyncCompleted;
-		
-		public IpodSyncTransaction(Device device)
-		{
-			this.device = device;
-			showCount = false;
-		}
-		
-		public IpodSyncTransaction(Device device, ArrayList tracks, 
-		  ArrayList removeTracks) : this(device)
-		{
-			updateTracks = tracks;
-			this.removeTracks = removeTracks;
-		}
-		
-		private bool ExistsOnIpod(Song[] songs, TrackInfo libTrack)
-		{
-			if(libTrack.GetType() == typeof(IpodTrackInfo)) {
-				if((libTrack as IpodTrackInfo).NeedSync) {
-					return false;
-				}
-			}
-		
-			foreach(Song song in songs) {
-				if(IpodMisc.TrackCompare(libTrack, song))
-					return true;
-			}
-			
-			return false;
-		}
-		
-		private bool ExistsInLibrary(Song song)
-		{
-			foreach(LibraryTrackInfo libTrack in Core.Library.Tracks.Values) {
-				if(IpodMisc.TrackCompare(libTrack, song))
-					return true;
-			}
-			
-			return false;
-		}
-		
-		protected override void CancelAction()
-		{
-		    encoder.Cancel();
-		}
-
-		private void EmitSyncStarted()
-		{
-			EventHandler handler = SyncStarted;
-			if(handler != null)
-				handler(this, new EventArgs());
-		}
-
-		private void EmitSyncCompleted()
-		{
-			EventHandler handler = SyncCompleted;
-			if(handler != null)
-				handler(this, new EventArgs());
-		}
-		
-		public override void Run()
-		{
-			EmitSyncStarted();
-			try {
-				WrapRun();
-			} catch(Exception e) {
-				Core.Log.Push(LogEntryType.UserError, 
-					Catalog.GetString("Error Syncing iPod"),
-					e.Message);
-			}
-			EmitSyncCompleted();
-		}
-		
-		public void WrapRun()
-		{
-			//statusMessage = String.Format(Catalog.GetString(
-			//	"Preparing to sync '{0}'"), device.Name);
-				
-			currentCount = 0;
-			totalCount = 0;
-			
-			bool doUpdate = false;
-			
-			profile = PipelineProfile.GetConfiguredProfile(
+    public class IpodSync
+    {
+        private PipelineProfile encodeProfile;
+        private Device device;
+        private ArrayList updateTracks;
+        private ArrayList removeTracks;
+        
+        private ProgressDialog progress;
+        
+        private Hashtable copyFiles;
+        
+        public event EventHandler SyncStarted;
+        public event EventHandler SyncCompleted;
+        
+        public IpodSync(Device device)
+        {
+            this.device = device;
+            
+            encodeProfile = PipelineProfile.GetConfiguredProfile(
                 "Ipod", "mp3,aac,mp4,m4a,m4p");
-            encoder = new GstFileEncoder();
-            encoder.Progress += OnEncodeProgress;
-			
-			if(updateTracks == null) {
-				foreach(Song song in device.SongDatabase.Songs) {
-					if(ExistsInLibrary(song))
-						continue;
+                
+           // progress = new ProgressDialog(null);
+           // progress.SongDatabase = device.SongDatabase;
+        }
+        
+        public IpodSync(Device device, ArrayList updateTracks, ArrayList removeTracks)
+            : this(device)
+        {
+            this.updateTracks = updateTracks;
+            this.removeTracks = removeTracks;
+        }
+        
+        public void StartSync()
+        {
+            EncodeFiles();
+        }
+        
+        private void EmitSyncStarted()
+        {
+            EventHandler handler = SyncStarted;
+            if(handler != null)
+                handler(this, new EventArgs());
+        }
 
-					device.SongDatabase.RemoveSong(song);
-					doUpdate = true;
-				}
-			} else if(removeTracks != null) {
-			    foreach(IpodTrackInfo libTrack in removeTracks) {
-			         device.SongDatabase.RemoveSong(libTrack.Song);
-			         doUpdate = true;
-			    }
-			}
-			
-			Song[] ipodSongs = device.SongDatabase.Songs;
-			
-			ICollection collection;
-			if(updateTracks == null)
-				collection = Core.Library.Tracks.Values;
-			else
-				collection = updateTracks;
-			
-			foreach(TrackInfo libTrack in collection) {
-			     if(cancelRequested)
-			         break;
-			         
-			    if(ExistsOnIpod(ipodSongs, libTrack) || libTrack.Uri == null)
-			    		continue;
-			         
-			    Song song = null;
-			         
-			    if(libTrack.GetType() == typeof(LibraryTrackInfo)) {
-				    song = IpodMisc.TrackInfoToSong(device, 
-				    	libTrack as LibraryTrackInfo);
-				} else if(libTrack.GetType() == typeof(IpodTrackInfo)) {
-					song = (libTrack as IpodTrackInfo).Song;
-				} else {
-					continue;
-				}
-				
-				if(song == null)
-					continue;
-				
-			    string filename = IpodCore.GetIpodSong(libTrack.Uri);
-			    if(filename == null) {
-			         if(profile == null) {
-			             continue;
-			         }
-			       
-			         filename = IpodCore.ConvertSongName(libTrack.Uri,
-			             profile.Extension);
-			         
-				  //  statusMessage = String.Format(
-				//	Catalog.GetString("Encoding for iPod Usage: {0} - {1}"),
-				//	song.Artist, song.Title);
-					
-					
-			        try {
-			        		encoder.Encode(libTrack.Uri, filename, profile);
-			        	} catch(Exception e) {
-			        		Core.Log.Push(LogEntryType.Warning,
-			        			Catalog.GetString("Could not encode file for iPod"),
-			        			String.Format("{0} -> {1} failed",
-			        				libTrack.Uri, filename));
-			        		filename = null;
-			        }
-			    }
-			    
-			    if(filename == null)
-			         continue;
+        private void EmitSyncCompleted()
+        {
+            EventHandler handler = SyncCompleted;
+            if(handler != null)
+                handler(this, new EventArgs());
+        }
+        
+        private bool ExistsOnIpod(Song[] songs, TrackInfo libTrack)
+        {
+            if(libTrack.GetType() == typeof(IpodTrackInfo)) {
+                if((libTrack as IpodTrackInfo).NeedSync) {
+                    return false;
+                }
+            }
 
-				try {
-					song.Filename = StringUtil.UriToFileName(filename);
-				} catch(Exception e) {
-					Core.Log.Push(LogEntryType.Warning,
-		        			Catalog.GetString("Could not transfer file to iPod"),
-		        			e.Message);
-					continue;
-				}
+            foreach(Song song in songs) {
+                if(IpodMisc.TrackCompare(libTrack, song))
+                    return true;
+            }
 
-				doUpdate = true;
-			}
-			
-			encoder.Dispose();
-			
-			if(!doUpdate || cancelRequested) {
-			    device.SongDatabase.Reload();
-				return;
-		    }
-			
-			//device.SongDatabase.SaveProgressChanged += OnSaveProgressChanged;
+            return false;
+        }
 
-			try {
-			    //statusMessage = Catalog.GetString("Synchronizing iPod...");
-			    //currentCount = 0;
-			    //totalCount = 0;
-				device.SongDatabase.Save();
-			} catch(Exception e) {
-				Core.Log.Push(LogEntryType.UserError, 
-					Catalog.GetString("Could not sync iPod"),
-					e.Message);
-			}
-			
+        private bool ExistsInLibrary(Song song)
+        {
+            foreach(LibraryTrackInfo libTrack in Core.Library.Tracks.Values) {
+                if(IpodMisc.TrackCompare(libTrack, song))
+                    return true;
+            }
 
-			//device.SongDatabase.SaveProgressChanged -= OnSaveProgressChanged;
-		} 
-		
-		/*private void OnSaveProgressChanged(SongDatabase db, Song song, 
-			double currentPercent, int completed, int total)
-		{
-			
-			currentCount = completed;
-			totalCount = total;
-			statusMessage = String.Format(Catalog.GetString(
-				"Copying {0} - {1}"), song.Artist, song.Title);
-		}*/
-		
-		private void OnEncodeProgress(object o, FileEncoderProgressArgs args)
-		{
-		    totalCount = 100;
-		    currentCount = (int)(100.0 * args.Progress);
-		}
-	}
-	
+            return false;
+        }
+        
+        private void EncodeFiles()
+        {
+            copyFiles = new Hashtable();
+            FileEncodeTransaction fet = null;
+            
+            Song [] ipodSongs = device.SongDatabase.Songs;
+            
+            ICollection collection = updateTracks;
+            if(collection == null)
+                collection = Core.Library.Tracks.Values;
+                
+            foreach(TrackInfo libTrack in collection) {                    
+                if(ExistsOnIpod(ipodSongs, libTrack) || libTrack.Uri == null)
+                    continue;
+
+                Song song = null;
+
+                if(libTrack is LibraryTrackInfo) {
+                    song = IpodMisc.TrackInfoToSong(device, libTrack as LibraryTrackInfo);
+                } else if(libTrack is IpodTrackInfo) {
+                    song = (libTrack as IpodTrackInfo).Song;
+                } else {
+                    continue;
+                }
+
+                if(song == null)
+                    continue;
+
+                string filename = IpodCore.GetIpodSong(libTrack.Uri);
+                
+                if(filename == null) {
+                    if(encodeProfile == null) {
+                        continue;
+                    }
+
+                    filename = IpodCore.ConvertSongName(libTrack.Uri, encodeProfile.Extension);
+                    Uri uri = new Uri(filename);
+                    
+                    copyFiles.Add(song, uri);
+                    
+                    if(fet == null) {
+                        fet = new FileEncodeTransaction(encodeProfile);
+                        fet.FileEncodeComplete += OnFileEncodeComplete;
+                        fet.Finished += OnFileEncodeBatchFinished;
+                    }
+                        
+                    fet.AddTrack(libTrack.Uri, uri);
+                } else {
+                    if(System.IO.File.Exists(filename)) {
+                        song.Filename = filename;
+                        copyFiles.Add(song, String.Empty);
+                    }
+                }
+            }
+            
+            if(fet != null) {
+                fet.Register();
+            } else {
+                SyncIpod();
+            }
+        }
+        
+        private void OnFileEncodeComplete(object o, FileEncodeCompleteArgs args)
+        {
+            foreach(Song song in copyFiles.Keys) {
+                if(copyFiles[song] == args.EncodedFileUri) {
+                    song.Filename = args.EncodedFileUri.LocalPath;
+                }
+            }
+        }
+        
+        private void OnFileEncodeBatchFinished(object o, EventArgs args)
+        {
+            SyncIpod();
+        }
+        
+        private void SyncIpod()
+        {
+            bool doUpdate = false;
+            
+            if(updateTracks == null) {
+                foreach(Song song in device.SongDatabase.Songs) {
+                    if(ExistsInLibrary(song))
+                        continue;
+
+                    device.SongDatabase.RemoveSong(song);
+                    doUpdate = true;
+                }
+            } else if(removeTracks != null) {
+                foreach(IpodTrackInfo libTrack in removeTracks) {
+                    device.SongDatabase.RemoveSong(libTrack.Song);
+                    doUpdate = true;
+                }
+            }
+            
+            foreach(Song song in copyFiles.Keys) {
+                if(song.Filename != null && song.Filename != String.Empty) {
+                    doUpdate = true;
+                    break;
+                }
+            }
+            
+            if(!doUpdate)
+                return;
+                
+            Thread thread = new Thread(new ThreadStart(ThreadedSave));
+            thread.Start();
+        }
+        
+        private void ThreadedSave()
+        {
+           try {
+                device.SongDatabase.Save();
+            
+           } catch(Exception e) {
+            Console.WriteLine(e);
+           }
+        }    
+    }
+    
 	public class IpodMisc
 	{
 		public static Song TrackInfoToSong(Device device, 
@@ -461,5 +436,27 @@ namespace Banshee
 				song.TrackNumber == libTrack.TrackNumber;
 		}
 		
+		public static string GetDiskUsageString(Device device)
+		{
+			// Translators: iPod disk usage. Each {N} is something like "100 MB"
+			return String.Format(
+				Catalog.GetString("{0} of {1}"),
+				Utilities.BytesToString(device.VolumeUsed),
+				Utilities.BytesToString(device.VolumeSize));
+		}
+
+		public static string GetDiskAvailableString(Device device)
+		{
+			// Translators: iPod disk usage. {0} is something like "100 MB"
+			return String.Format(
+				Catalog.GetString("({0} Remaining)"),
+				Utilities.BytesToString(device.VolumeAvailable));
+		}
+	   
+	    public static double GetDiskUsageFraction(Device device)
+	    {
+	       return (double)device.VolumeUsed /
+					(double)device.VolumeSize;
+	    }
 	}
 }
