@@ -41,15 +41,10 @@ namespace Banshee
 {
 	public class LibraryTrackInfo : TrackInfo
 	{
-		// This is correct; it's used for internal non-user-visible stuff
-		System.Globalization.CultureInfo ci = 
-			new System.Globalization.CultureInfo("en-US");
-	
-		public static int GetId(string lookup)
+		public static int GetId(Uri lookup)
 		{
 			Statement query = new Select("Tracks", new List("TrackID")) +
-				new Where(new Compare("Uri", Op.EqualTo, lookup), 
-					Op.Or, new Compare("TrackID", Op.EqualTo, lookup));
+				new Where(new Compare("Uri", Op.EqualTo, lookup));
 
 			try {
 				object result = Core.Library.Db.QuerySingle(query);
@@ -65,7 +60,7 @@ namespace Banshee
 			canSaveToDatabase = true;
 		}
 	
-        private void CheckIfExists(string uri)
+        private void CheckIfExists(Uri uri)
         {
               bool exists = false;
               try {
@@ -73,15 +68,92 @@ namespace Banshee
               } catch(Exception) {
                 exists = false;
               }
-              
+//              Console.WriteLine ("{0} {1}", uri.LocalPath, exists ? "exists" : "does not exist");
               if(exists) {
                   // TODO: we should actually probably take this as a hint to
                   // reparse metadata
                   throw new ApplicationException("Song is already in library");
               } 
         }
+
+		private void CheckIfExists(string filename)
+		{
+			CheckIfExists (new Uri ("file://" + filename));
+		}
+
+		private string MoveToPlace(string old_filename, bool initial_import)
+		{
+			bool in_library = old_filename.StartsWith (Core.Library.Location);
+//			Console.WriteLine ("\"{0}\" in \"{1}\": {2}", old_filename, Core.Library.Location, in_library);
+
+			if (initial_import && !in_library) {
+				bool copy = false;
+				try {
+					copy = (bool)Core.GconfClient.Get(GConfKeys.CopyOnImport);
+				} catch {}
+
+				if (copy) {
+					string new_filename = FileNamePattern.BuildFull(this,
+						Path.GetExtension (old_filename).Substring(1));
+					CheckIfExists(new_filename);
+
+					try {
+
+//						Console.WriteLine ("!in_library: {0}", new_filename);
+
+//						Console.WriteLine ("if (File.Exists (\"{0}\"): {1}", new_filename, File.Exists (new_filename));
+						if (File.Exists (new_filename))
+							return null;
+//						Console.WriteLine ("File.Copy(\"{0}\", \"{1}\", false);", old_filename, new_filename);
+						File.Copy(old_filename, new_filename, false);
+						return new_filename;
+					} catch {
+						return null;
+					}
+				}
+			}
+
+			if (in_library) {
+				bool move = false;
+
+				try {
+					move = (bool)Core.GconfClient.Get(GConfKeys.MoveOnInfoSave);
+				} catch {}
 	
-	    public LibraryTrackInfo(string uri, string artist, string album, 
+				if (move) {
+					string new_filename = FileNamePattern.BuildFull(this,
+						Path.GetExtension (old_filename).Substring(1));
+//					Console.WriteLine ("in_library: {0}", new_filename);
+					CheckIfExists(new_filename);
+
+					try {
+						if (File.Exists (new_filename))
+							return null;
+
+						if (old_filename != new_filename) {
+							// Move and set uri.
+							File.Move (old_filename, new_filename);
+	
+							// Delete old directories if empty.
+							try {
+								string old_dir = Path.GetDirectoryName (old_filename);
+								while (old_dir != null && old_dir != String.Empty) {
+									Directory.Delete (old_dir);
+									old_dir = Path.GetDirectoryName (old_dir);
+								}
+							} catch {}
+
+							return new_filename;
+						}
+					} catch {
+						return null;
+					}
+				}
+			}
+			return null;
+		}
+
+	    public LibraryTrackInfo(Uri uri, string artist, string album, 
 	       string title, string genre, uint trackNumber, uint trackCount,
 	       int year, long duration)
 	    {
@@ -99,37 +171,44 @@ namespace Banshee
 			this.year = year;
 			this.duration = duration;
 			
+			this.dateAdded = DateTime.Now;
+			
 			CheckIfExists(uri);
 			
 			SaveToDatabase(true);
 			Core.Library.SetTrack(trackId, this);
 			
-			uid = UidGenerator.Next;
 			PreviousTrack = Gtk.TreeIter.Zero;
 	    }
 	    
-        public LibraryTrackInfo(string uri, AudioCdTrackInfo track) : this(
+        public LibraryTrackInfo(Uri uri, AudioCdTrackInfo track) : this(
             uri, track.Artist, track.Album, track.Title, track.Genre,
             track.TrackNumber, track.TrackCount, track.Year, track.Duration)
         {
         
         }
 	
-		public LibraryTrackInfo(string uri) : this()
+		public LibraryTrackInfo(string filename) : this()
 		{
-			if(uri.StartsWith("sql://")) {
-				uri = uri.Substring(6);
-				LoadFromDatabase(uri);
-			} else {
-                CheckIfExists(uri);
-                
-            		if(!LoadFromDatabase(uri))
-					LoadFromFile(uri);
+//			Console.WriteLine ("LibraryTrackInfo(\"{0}\");", filename);
+			Uri old_uri = new Uri ("file://" + filename);
+
+			CheckIfExists(old_uri);
+			if(!LoadFromDatabase(old_uri)) {
+//				Console.WriteLine ("LoadFromFile(\"{0}\");", filename);
+				LoadFromFile(filename);
+
+				string new_filename = MoveToPlace(filename, true);
+
+				uri = new Uri ("file://" + (new_filename != null ? new_filename : filename));
+
+				CheckIfExists(uri);
+
+				SaveToDatabase(true);
 			}
-			
+
 			Core.Library.SetTrack(trackId, this);
-			
-			uid = UidGenerator.Next;
+
 			PreviousTrack = Gtk.TreeIter.Zero;
 		}
 		
@@ -137,27 +216,27 @@ namespace Banshee
 		{
 			LoadFromDatabaseReader(reader);
 			Core.Library.SetTrack(trackId, this);
-			uid = UidGenerator.Next;
 			PreviousTrack = Gtk.TreeIter.Zero;
 		}
 		
-		private void ParseUri(string uri)
+		private void ParseUri(string path)
 		{
-			artist = "";
-			album = "";
-			title = "";
+			artist = String.Empty;
+			album = String.Empty;
+			title = String.Empty;
 			trackNumber = 0;
 			Match match;
 
-			string fileName = StringUtil.UriEscape(uri);
+			string fileName = StringUtil.UriEscape(path);
 			fileName = Path.GetFileNameWithoutExtension(fileName);
 		
 			match = Regex.Match(fileName, @"(\d+)\.(.*)$");
 			if(match.Success) {
 				trackNumber = Convert.ToUInt32(match.Groups[1].ToString());
+//				Console.WriteLine ("trackNumber = {0}", trackNumber);
 				fileName = match.Groups[2].ToString().Trim();
 			}
-			
+
 			/* Artist - Album - Title */
 			match = Regex.Match(fileName, @"\s*(.*)-\s*(.*)-\s*(.*)$");
 			if(match.Success) {
@@ -175,6 +254,20 @@ namespace Banshee
 					title = fileName;
 				}
 			}
+
+			while (path != null && path != String.Empty) {
+				path = Path.GetDirectoryName(path);
+				fileName = Path.GetFileName (path);
+				if (album == String.Empty) {
+					album = fileName;
+					continue;
+				}
+				if (artist == String.Empty) {
+					artist = fileName;
+					continue;
+				}
+				break;
+			}
 			
 			artist = artist.Trim();
 			album = album.Trim();
@@ -191,7 +284,9 @@ namespace Banshee
 		private void SaveToDatabase(bool retryIfFail)
 		{
 			Statement tracksQuery;
-		
+
+//			Console.WriteLine ("{0} has id {1}", uri.LocalPath, TrackId);
+
 			if(trackId <= 0) {
 				tracksQuery = new Insert("Tracks", true,
 					"TrackID", null, 
@@ -205,7 +300,7 @@ namespace Banshee
 					"Title", title, 
 					"Genre", genre, 
 					"Year", year,
-					"DateAdded", dateAdded.ToString(ci.DateTimeFormat), 
+					"DateAddedStamp", Mono.Unix.Native.NativeConvert.FromDateTime(dateAdded), 
 					"TrackNumber", trackNumber, 
 					"TrackCount", trackCount, 
 					"Duration", duration, 
@@ -215,7 +310,7 @@ namespace Banshee
 					"AlbumPeak", albumPeak, 
 					"Rating", rating, 
 					"NumberOfPlays", numberOfPlays, 
-					"LastPlayed", lastPlayed.ToString(ci.DateTimeFormat));
+					"LastPlayedStamp", Mono.Unix.Native.NativeConvert.FromDateTime(lastPlayed));
 			} else {
 				tracksQuery = new Update("Tracks",
 					"Uri", uri, 
@@ -228,7 +323,7 @@ namespace Banshee
 					"Title", title, 
 					"Genre", genre, 
 					"Year", year,
-					"DateAdded", dateAdded.ToString(ci.DateTimeFormat), 
+					"DateAddedStamp", Mono.Unix.Native.NativeConvert.FromDateTime(dateAdded), 
 					"TrackNumber", trackNumber, 
 					"TrackCount", trackCount, 
 					"Duration", duration, 
@@ -238,19 +333,22 @@ namespace Banshee
 					"AlbumPeak", albumPeak, 
 					"Rating", rating, 
 					"NumberOfPlays", numberOfPlays, 
-					"LastPlayed", lastPlayed.ToString(ci.DateTimeFormat)) +
+					"LastPlayedStamp", Mono.Unix.Native.NativeConvert.FromDateTime(lastPlayed)) +
 					new Where(new Compare("TrackID", Op.EqualTo, trackId));// +
 				//	new Limit(1);
 			}
-			
+
 			Core.Library.Db.Execute(tracksQuery);
-			
+
 			/*if(Core.Library.Db.Execute(query) <= 0 && retryIfFail) {
 				trackId = 0;
 				SaveToDatabase(false);
 			} else if(trackId <= 0) {*/
 			
-			trackId = GetId(uri); /* OPTIMIZE! Seems like an unnecessary query */
+			if(trackId <= 0)
+			   trackId = GetId(uri); /* OPTIMIZE! Seems like an unnecessary query */
+
+//			Console.WriteLine ("{0} has id {1}", uri.LocalPath, TrackId);
 		}
 		
 		private bool LoadFromDatabase(object id)
@@ -275,68 +373,51 @@ namespace Banshee
 			return true;
 		}
 		
-		private void LoadFromDatabaseReader(IDataReader reader)
-		{
-			Hashtable colmap = Core.Library.Db.ColumnMap("Tracks");
-			
-			trackId = Convert.ToInt32(reader[(int)colmap["TrackID"]]);
-				
-			uri = (string)reader[(int)colmap["Uri"]];
-			mimetype = (string)reader[(int)colmap["MimeType"]];
-			
-			album = (string)reader[(int)colmap["AlbumTitle"]];
-			artist = (string)reader[(int)colmap["Artist"]];
-			performer = (string)reader[(int)colmap["Performer"]];
-			title = (string)reader[(int)colmap["Title"]];
-			genre = (string)reader[(int)colmap["Genre"]];
-			dateAdded = DateTime.Parse((string)reader[(int)colmap["DateAdded"]], 
-				ci.DateTimeFormat);
-			year = Convert.ToInt32(reader[(int)colmap["Year"]]);
-			
-			trackNumber = Convert.ToUInt32(reader[(int)colmap["TrackNumber"]]);
-			trackCount = Convert.ToUInt32(reader[(int)colmap["TrackCount"]]);
-			duration = Convert.ToInt64(reader[(int)colmap["Duration"]]);
-			rating = Convert.ToUInt32(reader[(int)colmap["Rating"]]);
-			numberOfPlays = 
-				Convert.ToUInt32(reader[(int)colmap["NumberOfPlays"]]);
-			lastPlayed = 
-				DateTime.Parse((string)reader[(int)colmap["LastPlayed"]], 
-				ci.DateTimeFormat);
-				
-			/*if(reader == null)
-				return;
-				
-			foreach(string key in reader.GetSchemaTable().Columns) 
-				Console.WriteLine(key);*/
-				
-			/*trackId = Convert.ToInt32(reader["TrackID"]);
-				
-			uri = (string)reader["Uri"];
-			mimetype = (string)reader["MimeType"];
-			
-			artist = (string)reader["Artist"];
-			performer = (string)reader["Performer"];
-			title = (string)reader["Title"];
-			genre = (string)reader["Genre"];
-			date = DateTime.Parse((string)reader["Date"]);
-			
-			trackNumber = Convert.ToUInt32(reader["TrackNumber"]);
-			trackCount = Convert.ToUInt32(reader["TrackCount"]);
-			duration = Convert.ToInt64(reader["Duration"]);
-			rating = Convert.ToUInt32(reader["Rating"]);
-			numberOfPlays = 
-				Convert.ToUInt32(reader["NumberOfPlays"]);
-			lastPlayed = 
-				DateTime.Parse((string)reader["LastPlayed"]);*/
+        private void LoadFromDatabaseReader(IDataReader reader)
+        {
+            trackId = Convert.ToInt32(reader["TrackID"]);
+
+            try {
+                uri = new Uri(reader["Uri"] as string);
+            } catch(UriFormatException) {
+                uri = new Uri("file://" + (reader["Uri"] as string));
+            }
+            
+            mimetype = reader["MimeType"] as string;
+
+            album = reader["AlbumTitle"] as string;
+            artist = reader["Artist"] as string;
+            performer = reader["Performer"] as string;
+            title = reader["Title"] as string;
+            genre = reader["Genre"] as string;
+
+            year = Convert.ToInt32(reader["Year"]);
+
+            trackNumber = Convert.ToUInt32(reader["TrackNumber"]);
+            trackCount = Convert.ToUInt32(reader["TrackCount"]);
+            duration = Convert.ToInt64(reader["Duration"]);
+            rating = Convert.ToUInt32(reader["Rating"]);
+            numberOfPlays = Convert.ToUInt32(reader["NumberOfPlays"]);
+
+            try {
+                lastPlayed = Mono.Unix.Native.NativeConvert.ToDateTime((long)reader["LastPlayedStamp"]);
+            } catch(Exception) {
+                lastPlayed = DateTime.MinValue;
+            }
+
+            try {
+                dateAdded = Mono.Unix.Native.NativeConvert.ToDateTime((long)reader["DateAddedStamp"]);
+            } catch(Exception) {
+                dateAdded = DateTime.MinValue;
+            }
 		}
 		
-		private void LoadFromFile(string uri)
+		private void LoadFromFile(string filename)
 		{
-			this.uri = uri;
-			ParseUri(uri);
+			ParseUri(filename);
 			trackId = 0;
 	
-			AudioFileWrapper af = new AudioFileWrapper(uri);
+			AudioFileWrapper af = new AudioFileWrapper(filename);
 
 			mimetype = null;
 
@@ -344,17 +425,34 @@ namespace Banshee
 			album = af.Album == null ? album : af.Album;
 			title = af.Title == null ? title : af.Title;
 			genre = af.Genre == null ? genre : af.Genre;
-			trackNumber = (uint)af.TrackNumber;
+			trackNumber = af.TrackNumber == 0 ? trackNumber : (uint)af.TrackNumber;
 			trackCount = 0;
 			duration = af.Duration;
 			year = af.Year;
 			
-			SaveToDatabase(true);
+			this.dateAdded = DateTime.Now;
 		}
-		
+
 		public override void Save()
 		{
-			SaveToDatabase(true);
+			try {
+				string new_filename = MoveToPlace (uri.LocalPath, false);
+				if (new_filename != null) {
+					this.uri = new Uri ("file://" + new_filename);
+				}
+			} catch {}
+
+			try {
+				Core.Library.Db.WriteCycleFinished -= OnDbWriteCycleFinished;
+				SaveToDatabase(true);
+			} catch {
+				Core.Library.Db.WriteCycleFinished += OnDbWriteCycleFinished;
+			}
+		}
+		
+		private void OnDbWriteCycleFinished(object o, EventArgs args)
+		{
+		    Save();
 		}
 		
 		public override void IncrementPlayCount()
@@ -362,21 +460,24 @@ namespace Banshee
 			numberOfPlays++;
 			lastPlayed = DateTime.Now;
 			
-			Statement query = new Update("Tracks",
+			/*Statement query = new Update("Tracks",
 				"NumberOfPlays", numberOfPlays, 
 				"LastPlayed", lastPlayed.ToString(ci.DateTimeFormat)) +
 				new Where(new Compare("TrackID", Op.EqualTo, trackId));
 				//new Limit(1);
 
-			Core.Library.Db.Execute(query);
+			Core.Library.Db.Execute(query);*/
+			
+			Save();
 		}
 		
 		protected override void SaveRating()
 		{
-			Statement query = new Update("Tracks",
+			/*Statement query = new Update("Tracks",
 				"Rating", rating) +
 				new Where(new Compare("TrackID", Op.EqualTo, trackId));
-			Core.Library.Db.Execute(query);
+			Core.Library.Db.Execute(query);*/
+			Save();
 		}
 	}
 }
