@@ -31,7 +31,7 @@ using System;
 using System.Runtime.InteropServices;
 using Gtk;
 using Gdk;
-using Mono.Posix;
+using Mono.Unix;
 
 namespace Banshee
 {
@@ -90,7 +90,7 @@ namespace Banshee
 					menu_y = (int)args.Event.YRoot - (int)args.Event.Y;
 					traymenu.Popup(null, null, 
 						new MenuPositionFunc(PositionMenu), 
-						IntPtr.Zero, args.Event.Button, args.Event.Time);
+						args.Event.Button, args.Event.Time);
 					break;
 			}
 
@@ -403,7 +403,7 @@ namespace Banshee
 	internal struct XClientMessageEvent
 	{
 		internal XEventName     type;
-		internal int            serial;
+		internal IntPtr         serial;
 		internal bool           send_event;
 		internal IntPtr         display;
 		internal IntPtr         window;
