@@ -30,7 +30,7 @@
 using System;
 using System.Collections;
 using Gtk;
-using Mono.Posix;
+using Mono.Unix;
 
 namespace Banshee
 {
